using System;
using System.Threading;

namespace DiningPhilosophersSample
{
    public partial class Philosopher
    {
        public void Init( Philosopher leftNeighbour, Philosopher rightNeighbour,
                            Fork leftFork, Fork rightFork,
                            bool takeLeftFork, bool takeRightFork )
        {
            if ( null == leftNeighbour )
                throw new ArgumentNullException( "leftNeighbour" );
            if ( null == rightNeighbour )
                throw new ArgumentNullException( "rightNeighbour" );
            this.leftNeighbour = leftNeighbour;
            this.rightNeighbour = rightNeighbour;

            if ( null == leftFork )
                throw new ArgumentNullException( "leftFork" );
            if ( null == rightFork )
                throw new ArgumentNullException( "rightFork" );
            this.leftFork = leftFork;
            this.rightFork = rightFork;

            if ( takeLeftFork )
                this.leftFork.Owner = this;
            if ( takeRightFork )
                this.rightFork.Owner = this;
        }

        private Philosopher leftNeighbour;
        private Philosopher rightNeighbour;

        private Fork leftFork;
        private Fork rightFork;

        private bool HasLFork
        {
            get { return ( this == leftFork.Owner ); }
        }

        private bool HasRFork
        {
            get { return ( this == rightFork.Owner ); }
        }

        public delegate void EmptyEvent();

        public delegate void PhilosopherEvent( Philosopher philosopher );

        public event PhilosopherEvent WantToEat;

        public event PhilosopherEvent Satiated;

        public event EmptyEvent LeftForkQuery;

        public event EmptyEvent RightForkQuery;

        public event EmptyEvent LeftForkGiven;

        public event EmptyEvent RightForkGiven;

        private const int maxSleep = 1000;

        private Random random = new Random();

        public void Think()
        {
            Thread.Sleep( random.Next( maxSleep ) );

            WantToEat( this );
        }

        public void GiveLeftFork()
        {
            if ( !HasLFork )
                throw new InvalidOperationException( "Philosopher has not LFork!" );

            if ( leftNeighbour == leftFork.Owner )
                throw new InvalidOperationException( "Left Neighbour already has RFork!" );

            leftFork.Owner = leftNeighbour;

            LeftForkGiven();
        }


        public void GiveRightFork()
        {
            if ( !HasRFork )
                throw new InvalidOperationException( "Philosopher has not RFork!" );

            if ( rightNeighbour == rightFork.Owner )
                throw new InvalidOperationException( "Right Neighbour already has LFork!" );

            rightFork.Owner = rightNeighbour;

            RightForkGiven();
        }

        public void Eat()
        {
            if ( !HasLFork && !HasRFork )
                throw new InvalidOperationException( "Philosopher cannot eat without Forks!" );
            if ( !HasLFork )
                throw new InvalidOperationException( "Philosopher cannot eat without LFork!" );
            if ( !HasRFork )
                throw new InvalidOperationException( "Philosopher cannot eat without RFork!" );

            Thread.Sleep( random.Next( maxSleep ) );

            Satiated( this );
        }
    }
}