using System;
using System.Collections.Generic;
using System.Threading;

namespace DiningPhilosophersSample
{
    public partial class Philosopher
    {
        public void Init(   Fork    leftFork,       Fork    rightFork,
                            bool    takeLeftFork,   bool    takeRightFork )
        {
            if ( null == leftFork )
                throw new ArgumentNullException( "leftFork" );
            if ( null == rightFork )
                throw new ArgumentNullException( "rightFork" );

            forks.Add( leftFork );
            forks.Add( rightFork );

            if ( takeLeftFork )
                leftFork.Owner = this;
            if ( takeRightFork )
                rightFork.Owner = this;
        }

        private List<Fork> forks = new List<Fork>();

        private List<KeyValuePair<Philosopher, Fork>> forksToGive = new List<KeyValuePair<Philosopher, Fork>>();

        private bool HasAllForks
        {
            get
            {
                foreach ( Fork fork in forks )
                {
                    if ( this != fork.Owner )
                        return false;
                }

                return true;
            }
        }

        private void QueryForks()
        {
            foreach ( Fork fork in forks )
                QueryFork( fork );
        }

        private void GiveFork( Philosopher philosopher, Fork fork )
        {
            if ( this != fork.Owner )
                throw new InvalidOperationException( "Philosopher has not such fork!" );

            if ( fork.Clean )
            {
                forksToGive.Add( new KeyValuePair<Philosopher,Fork>( philosopher, fork ) );
            }
            else
            {
                fork.Clean = true;
                fork.Owner = philosopher;
                philosopher.ForkGot();
            }
        }

        private void QueryFork( Fork fork )
        {
            if ( this != fork.Owner )
                fork.Owner.ForkQueried( this, fork );
        }

        private void GiveQueriedForks()
        {
            foreach ( KeyValuePair<Philosopher, Fork> philosopherAndFork in forksToGive )
                GiveFork( philosopherAndFork.Key, philosopherAndFork.Value );

            forksToGive.Clear();
        }


        public delegate void EmptyEvent();

        public delegate void PhilosopherEvent( Philosopher philosopher );

        public delegate void PhilosopherAndForkEvent( Philosopher philosopher, Fork fork );

        public event PhilosopherEvent WantToEat;

        public event PhilosopherEvent Satiated;

        public event PhilosopherAndForkEvent ForkQueried;

        public event EmptyEvent ForkGot;

        private const int maxSleep = 1000;

        private Random random = new Random();

        public void Think()
        {
            Thread.Sleep( random.Next( maxSleep ) );

            WantToEat( this );
        }

        public void Eat()
        {
            if ( !HasAllForks )
                throw new InvalidOperationException( "Philosopher cannot eat without Forks!" );

            Thread.Sleep( random.Next( maxSleep ) );

            foreach ( Fork fork in forks )
            {
                fork.Clean = false;
            }

            Satiated( this );
        }
    }
}