using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.Design;
using ImTryin.StateMachineDesigner.DomainModel;
using Microsoft.VisualStudio.EnterpriseTools.Shell;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Utilities;

namespace ImTryin.StateMachineDesigner.Designer
{
    /// <summary>
    /// HT002 - add menu commands.
    /// 
    /// </summary>
    internal partial class CommandSet
    {
        protected override void GetMenuCommands( IList commandList )
        {
            // Call the base class method so that the
            // standard DSL menus get added.
            base.GetMenuCommands( commandList );

            MenuCommand cmd;

            cmd = new DynamicStatusMenuCommand( OnDiagramSelectionMenuStatus, OnShowHideGridMenuClick,
                                                new CommandID( GuidList.guidStateMachineLanguageCmdSet,
                                                               (int) PackageCommandIdList.cmdIdShowHideGridCommand ) );
            commandList.Add( cmd );

            cmd = new DynamicStatusMenuCommand( OnDiagramSelectionMenuStatus, OnExportDiagramAsImageMenuClick,
                                                new CommandID( GuidList.guidStateMachineLanguageCmdSet,
                                                               (int)
                                                               PackageCommandIdList.cmdIdExportDiagramAsImageCommand ) );
            commandList.Add( cmd );

            cmd = new DynamicStatusMenuCommand( OnConnectorsSelectionMenuStatus, OnFlipConnectorTextMenuClick,
                                                new CommandID( GuidList.guidStateMachineLanguageCmdSet,
                                                               (int) PackageCommandIdList.cmdIdFlipConnectorTextCommand ) );
            commandList.Add( cmd );

            cmd = new DynamicStatusMenuCommand( OnCodeGenerationSelectionMenuStatus, OnPurgeCodeGenerationMenuClick,
                                                new CommandID( GuidList.guidStateMachineLanguageCmdSet,
                                                               (int)
                                                               PackageCommandIdList.cmdIdPurgeCodeGenerationCommand ) );
            commandList.Add( cmd );
        }

        internal void OnDiagramSelectionMenuStatus( object sender, EventArgs e )
        {
            MenuCommand cmd = sender as MenuCommand;
            if ( null != cmd )
                cmd.Visible = cmd.Enabled = IsDiagramSelected();
        }

        internal void OnShowHideGridMenuClick( object sender, EventArgs e )
        {
            CurrentDocView.CurrentDiagram.ShowGrid = !CurrentDocView.CurrentDiagram.ShowGrid;
        }

        internal void OnExportDiagramAsImageMenuClick( object sender, EventArgs e )
        {
            CurrentDocView.CurrentDiagram.CreateMetafile(
                new object[] { CurrentDocView.CurrentDiagram }, CurrentDocView.DocData.FileName + ".emf" );
        }


        internal void OnConnectorsSelectionMenuStatus( object sender, EventArgs e )
        {
            MenuCommand cmd = sender as MenuCommand;
            if ( null != cmd )
            {
                cmd.Visible = cmd.Enabled = false;
                foreach ( object obj in CurrentSelection )
                {
                    if ( null != obj as Connector )
                    {
                        cmd.Visible = cmd.Enabled = true;
                        break;
                    }
                }
            }
        }

        internal void OnFlipConnectorTextMenuClick( object sender, EventArgs e )
        {
            using (
                Transaction t =
                    CurrentDocView.CurrentDiagram.TransactionManager.BeginTransaction( "Flip Connector Text" ) )
            {
                foreach ( object obj in CurrentSelection )
                {
                    Connector onnector = obj as Connector;
                    if ( null != onnector )
                    {
                        foreach ( ConnectorDecorator decorator in onnector.Decorators )
                        {
                            switch ( decorator.Position )
                            {
                                case ConnectorDecoratorPosition.SourceTop:
                                    decorator.Position = ConnectorDecoratorPosition.SourceBottom;
                                    break;
                                case ConnectorDecoratorPosition.SourceBottom:
                                    decorator.Position = ConnectorDecoratorPosition.SourceTop;
                                    break;
                                case ConnectorDecoratorPosition.TargetTop:
                                    decorator.Position = ConnectorDecoratorPosition.TargetBottom;
                                    break;
                                case ConnectorDecoratorPosition.TargetBottom:
                                    decorator.Position = ConnectorDecoratorPosition.TargetTop;
                                    break;
                                case ConnectorDecoratorPosition.CenterTop:
                                    decorator.Position = ConnectorDecoratorPosition.CenterBottom;
                                    break;
                                case ConnectorDecoratorPosition.CenterBottom:
                                    decorator.Position = ConnectorDecoratorPosition.CenterTop;
                                    break;
                            }
                        }

                        onnector.RipUp();
                    }
                }

                // Don't forget to commit the transaction.
                t.Commit();
            }
        }

        internal void OnCodeGenerationSelectionMenuStatus( object sender, EventArgs e )
        {
            MenuCommand cmd = sender as MenuCommand;
            if ( null != cmd )
            {
                cmd.Visible = cmd.Enabled = false;
                if ( IsSingleSelection() )
                {
                    Shape shape = SingleSelection as Shape;
                    if ( ( null != shape ) && ( null != shape.ModelElement as CodeGeneration ) )
                        cmd.Visible = cmd.Enabled = true;
                }
            }
        }

        internal void OnPurgeCodeGenerationMenuClick( object sender, EventArgs e )
        {
            CodeGeneration codeGeneration = ( (StateMachine) CurrentDocView.CurrentDiagram.ModelElement ).CodeGeneration;

            using ( Transaction t = codeGeneration.Store.TransactionManager.BeginTransaction( "Purge CodeGeneration" ) )
            {
                List< ModelElement > elemetsToRemove = new List< ModelElement >();

                foreach ( Event oEvent in codeGeneration.Events )
                {
                    if ( 0 == oEvent.Transitions.Count )
                        elemetsToRemove.Add( oEvent );
                }

                foreach ( Condition condition in codeGeneration.Conditions )
                {
                    if ( 0 == condition.Transitions.Count )
                        elemetsToRemove.Add( condition );
                }

                foreach ( Statement statement in codeGeneration.Statements )
                {
                    if ( ( 0 == statement.Transitions.Count ) && ( 0 == statement.StateActivities.Count ) )
                        elemetsToRemove.Add( statement );
                }

                foreach ( ModelElement element in elemetsToRemove )
                    element.Remove();

                // Don't forget to commit the transaction.
                t.Commit();
            }
        }
    }
}