using System;
using ImTryin.StateMachineDesigner.DomainModel;
using Microsoft.VisualStudio.Modeling;

namespace ImTryin.StateMachineDesigner.Designer
{
    [RuleOn( typeof ( StartState ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStartStateAddRule : AddRule
    {
        public override void ElementAdded( ElementAddedEventArgs e )
        {
            StartState startState = (StartState) e.ModelElement;
            if ( null == startState.StateMachine.StartState )
                startState.StateMachine.StartState = startState;
            else if ( startState != startState.StateMachine.StartState )
                throw new InvalidOperationException( "Only one Start State in State Machine is permitted!" );
        }
    }

    [RuleOn( typeof ( StartState ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStartStateRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            StartState startState = (StartState) e.ModelElement;
            if ( startState == startState.StateMachine.StartState )
                startState.StateMachine.StartState = null;
        }
    }

    internal static partial class GeneratedMetaModelTypes
    {
        internal static Type DomainModelStartStateAddRuleType =
            typeof ( DomainModelStartStateAddRule );

        internal static Type DomainModelStartStateRemovingRuleType =
            typeof ( DomainModelStartStateRemovingRule );
    }
}