using System;
using ImTryin.StateMachineDesigner.DomainModel;
using Microsoft.VisualStudio.Modeling;

namespace ImTryin.StateMachineDesigner.Designer
{
    [RuleOn( typeof ( StateMachine ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStateMachineAddRule : AddRule
    {
        public override void ElementAdded( ElementAddedEventArgs e )
        {
            StateMachine stateMachine = (StateMachine) e.ModelElement;
            if ( null == stateMachine.CodeGeneration )
                stateMachine.CodeGeneration = CodeGeneration.CreateCodeGeneration( stateMachine.Store );
        }
    }

    [RuleOn( typeof ( CodeGeneration ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelCodeGenerationRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            CodeGeneration codeGeneration = (CodeGeneration) e.ModelElement;
            if ( !codeGeneration.StateMachine.IsRemoving )
                throw new InvalidOperationException( "Remove CodeGeneration operation is disabled!" );
        }
    }

    internal static partial class GeneratedMetaModelTypes
    {
        internal static Type DomainModelStateMachineAddRuleType =
            typeof ( DomainModelStateMachineAddRule );

        internal static Type DomainModelCodeGenerationRemovingRuleType =
            typeof ( DomainModelCodeGenerationRemovingRule );
    }
}