using System;
using ImTryin.StateMachineDesigner.DomainModel;
using Microsoft.VisualStudio.Modeling;

namespace ImTryin.StateMachineDesigner.Designer
{
    [RuleOn( typeof ( State ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStateAddRule : AddRule
    {
        public override void ElementAdded( ElementAddedEventArgs e )
        {
            State state = (State) e.ModelElement;
            if ( 3 != state.Activities.Count )
            {
                state.Activities.Clear();

                StateActivity entryStateActivity = StateActivity.CreateStateActivity( state.Store );
                entryStateActivity.Name = "Entry Activity";
                entryStateActivity.Type = StateActivityType.Entry;
                entryStateActivity.State = state;

                StateActivity includeStateActivity = StateActivity.CreateStateActivity( state.Store );
                includeStateActivity.Name = "Include Activity";
                includeStateActivity.Type = StateActivityType.Include;
                includeStateActivity.State = state;

                StateActivity exitStateActivity = StateActivity.CreateStateActivity( state.Store );
                exitStateActivity.Name = "Exit Activity";
                exitStateActivity.Type = StateActivityType.Exit;
                exitStateActivity.State = state;
            }
        }
    }

    [RuleOn( typeof ( StateHasActivities ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStateHasActivitiesAddRule : AddRule
    {
        public override void ElementAdded( ElementAddedEventArgs e )
        {
            StateHasActivities stateHasActivities = (StateHasActivities) e.ModelElement;
            if ( 3 < stateHasActivities.State.Activities.Count )
                throw new InvalidOperationException( "StateActivity Add/Remove operations are disabled!" );
        }
    }

    [RuleOn( typeof ( StateHasActivities ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStateHasActivitiesRemovingRule : RemovingRule
    {
        public override void ElementRemoving( ElementRemovingEventArgs e )
        {
            StateHasActivities stateHasActivities = (StateHasActivities) e.ModelElement;
            if ( ( !stateHasActivities.State.IsRemoving ) && ( 3 >= stateHasActivities.State.Activities.Count ) )
                throw new InvalidOperationException( "StateActivity Add/Remove operations are disabled!" );
        }
    }

    [RuleOn( typeof ( StateActivity ), FireTime = TimeToFire.TopLevelCommit )]
    public sealed class DomainModelStateActivityChangeRule : ChangeRule
    {
        public override void ElementAttributeChanged( ElementAttributeChangedEventArgs e )
        {
            StateActivity stateActivity = (StateActivity) e.ModelElement;

            if ( ( e.MetaAttribute.Id == StateActivity.ActivityMetaAttributeGuid ) &&
                 ( ( StateActivityType.Entry == stateActivity.Type ) ||
                   ( StateActivityType.Exit == stateActivity.Type ) ) )
            {
                stateActivity.Statement =
                    stateActivity.State.StateMachine.CodeGeneration.GetStatement( stateActivity.Activity );
            }

            if ( ( e.MetaAttribute.Id == StateActivity.ActivityMetaAttributeGuid ) ||
                 ( e.MetaAttribute.Id == StateActivity.TypeMetaAttributeGuid ) )
                stateActivity.Notation = stateActivity.Type.ToString().ToLower() + "/" + stateActivity.Activity;
        }
    }

    internal static partial class GeneratedMetaModelTypes
    {
        internal static Type DomainModelStateAddRuleType =
            typeof ( DomainModelStateAddRule );

        internal static Type DomainModelStateHasActivitiesAddRuleType =
            typeof ( DomainModelStateHasActivitiesAddRule );

        internal static Type DomainModelStateHasActivitiesRemovingRuleType =
            typeof ( DomainModelStateHasActivitiesRemovingRule );

        internal static Type DomainModelStateActivityChangeRuleType =
            typeof ( DomainModelStateActivityChangeRule );
    }
}