using System.Collections.Generic;
using ImTryin.StateMachineDesigner.DomainModel.Validation;
using Microsoft.VisualStudio.Modeling.Validation;

namespace ImTryin.StateMachineDesigner.DomainModel
{
    [ValidationState( ValidationState.Enabled )]
    public partial class BaseState
    {
        [ValidationMethod( ValidationCategory.Menu | ValidationCategory.Open | ValidationCategory.Save )]
        private void ValidateStateOutgoingTransitions( ValidationContext context )
        {
            Dictionary< string, List< Transition > > transitionsGroups = new Dictionary< string, List< Transition > >();
            foreach ( Transition transition in GetElementLinks( Transition.TransitionFromMetaRoleGuid ) )
            {
                if ( !transitionsGroups.ContainsKey( transition.Trigger ) )
                    transitionsGroups.Add( transition.Trigger, new List< Transition >() );

                transitionsGroups[ transition.Trigger ].Add( transition );
            }

            foreach ( List< Transition > transitions in transitionsGroups.Values )
            {
                if ( 1 < transitions.Count )
                {
                    List< string > guards = new List< string >( transitions.Count );

                    foreach ( Transition transition in transitions )
                    {
                        if ( null == transition.Condition )
                        {
                            context.LogError( "Transitions with same triggers must be guarded", "NotGuardedSameTriggers",
                                              transition );
                        }

                        guards.Add( transition.Guard );
                    }

                    if ( !Artefacts.CheckOrthogonalConditions( guards, StateMachine.CodeGeneration ) )
                    {
                        context.LogError( "State has transitions with same triggers and not orthogonal guards",
                                          "NotOrthogonalGuards", transitions.ToArray() );
                    }
                }
            }
        }
    }
}