using ImTryin.StateMachineDesigner.DomainModel.Validation;
using Microsoft.VisualStudio.Modeling.Validation;

namespace ImTryin.StateMachineDesigner.DomainModel
{
    [ValidationState( ValidationState.Enabled )]
    public partial class CodeGeneration
    {
        [ValidationMethod( ValidationCategory.Menu | ValidationCategory.Open | ValidationCategory.Save )]
        private void ValidateCodeGeneration( ValidationContext context )
        {
            if ( "" == Namespace )
                context.LogError( "Please, provide Target Namespace for State Machine", "EmptyNamespace", this );
            else if ( !Artefacts.CheckNamespaceName( Namespace, this ) )
                context.LogError( "Target Namespace is invalid", "InvalidNamespace", this );


            if ( "" == ClassModifiers )
                context.LogWarning( "Target Class Modifiers is empty", "EmptyClassModifiers", this );
            else if ( !Artefacts.CheckClassModifiers( ClassModifiers, this ) )
                context.LogError( "Target Class Modifiers is invalid", "InvalidClassModifiers", this );

            if ( "" == Class )
                context.LogError( "Please, provide Target Class Name for State Machine", "EmptyClass", this );
            else if ( !Artefacts.CheckClassName( Class, this ) )
                context.LogError( "Target Class Name is invalid", "InvalidClass", this );

            if ( Synchronization.Async_OwnThread == EventHandling )
            {
                if ( "" == InitMethodModifiers )
                    context.LogWarning( "Init Method Modifiers is empty", "EmptyInitMethodModifiers", this );
                else if ( !Artefacts.CheckMethodModifiers( InitMethodModifiers, this ) )
                    context.LogError( "Init Method Modifiers is invalid", "InvalidInitMethodModifiers", this );

                if ( "" == InitMethod )
                    context.LogError( "Please, provide Init Method Name for State Machine", "EmptyInitMethod", this );
                else if ( !Artefacts.CheckMethodName( InitMethod, this ) )
                    context.LogError( "Init Method Name is invalid", "InvalidInitMethod", this );
            }

            if ( "" == StartMethodModifiers )
                context.LogWarning( "Start Method Modifiers is empty", "EmptyStartMethodModifiers", this );
            else if ( !Artefacts.CheckMethodModifiers( StartMethodModifiers, this ) )
                context.LogError( "Start Method Modifiers is invalid", "InvalidStartMethodModifiers", this );

            if ( "" == StartMethod )
                context.LogError( "Please, provide Start Method Name for State Machine", "EmptyStartMethod", this );
            else if ( !Artefacts.CheckMethodName( StartMethod, this ) )
                context.LogError( "Start Method Name is invalid", "InvalidStartMethod", this );


            if ( "" == JoinMethodModifiers )
                context.LogWarning( "Join Method Modifiers is empty", "EmptyJoinMethodModifiers", this );
            else if ( !Artefacts.CheckMethodModifiers( JoinMethodModifiers, this ) )
                context.LogError( "Join Method Modifiers is invalid", "InvalidJoinMethodModifiers", this );

            if ( "" == JoinMethod )
                context.LogError( "Please, provide Join Method Name for State Machine", "EmptyJoinMethod", this );
            else if ( !Artefacts.CheckMethodName( JoinMethod, this ) )
                context.LogError( "Join Method Name is invalid", "InvalidJoinMethod", this );


            if ( "" == AbortMethodModifiers )
                context.LogWarning( "Abort Method Modifiers is empty", "EmptyAbortMethodModifiers", this );
            else if ( !Artefacts.CheckMethodModifiers( AbortMethodModifiers, this ) )
                context.LogError( "Abort Method Modifiers is invalid", "InvalidAbortMethodModifiers", this );

            if ( "" == AbortMethod )
                context.LogError( "Please, provide Abort Method Name for State Machine", "EmptyAbortMethod", this );
            else if ( !Artefacts.CheckMethodName( AbortMethod, this ) )
                context.LogError( "Abort Method Name is invalid", "InvalidAbortMethod", this );


            foreach ( Event oEvent in Events )
            {
                if ( !Artefacts.CheckEventAccess( oEvent.Access, this ) )
                    context.LogError( "Wrong Event Access", "WrongEventAccess", oEvent );
                if ( !Artefacts.CheckParams( oEvent.Parameters, this ) )
                    context.LogError( "Wrong Event Parameters", "WrongEventParameters", oEvent );
            }

            foreach ( Condition condition in Conditions )
            {
                if ( !Artefacts.CheckCondition( condition.Notation, this ) )
                    context.LogError( "Wrong Condition", "WrongCondition", condition );
            }

            foreach ( Statement statement in Statements )
            {
                if ( !Artefacts.CheckStatement( statement.Notation, this ) )
                    context.LogError( "Wrong Statement", "WrongStatement", statement );
            }
        }
    }
}